@@ -13,8 +13,9 @@ from dataclasses import dataclass
 
 @dataclass
 class ReviewOutput:
-    review: str
+    topics: dict
     usage: dict
+    model_name: str
     cost: dict
 
 
@@ -22,6 +23,7 @@ class ReviewOutput:
 class DescOutput:
     desc: str
     usage: dict
+    model_name: str
     cost: dict
 
 
@@ -30,6 +32,19 @@ class CodeReviewer:
         self.logger = logging.getLogger(__name__)
         self.provider = LLMProvider(system_prompt=CODE_REVIEW_SYSTEM_PROMPT)
 
+    def is_code_review_prompt_within_limit(
+        self,
+        diff_text: str,
+        pull_request_title: str,
+        pull_request_desc: str,
+    ) -> bool:
+        prompt = CODE_REVIEW_PROMPT.format(
+            PULL_REQUEST_TITLE=pull_request_title,
+            PULL_REQUEST_DESC=pull_request_desc,
+            CODE_DIFF=diff_text,
+        )
+        return self.provider.is_inside_token_limit(PROMPT=prompt)
+
     def review_pull_request(
         self,
         diff_text: str,
@@ -37,7 +52,7 @@ class CodeReviewer:
         pull_request_desc: str,
         pull_request_files: List[Dict],
         user: Optional[str] = None,
-    ):
+    ) -> ReviewOutput:
 
         # If diff_text is smaller than 70% of model token
         prompt = CODE_REVIEW_PROMPT.format(
@@ -57,9 +72,12 @@ class CodeReviewer:
             # We recurrsively get feedback for files and then get basic summary
             reviews = []
             for file in pull_request_files:
-                patch_details = file["patch"]
-                filename = file["filename"]
-                if filename.split(".")[-1] not in parser.EXCLUDED_FILETYPES:
+                patch_details = file.get("patch")
+                filename = file.get("filename", "")
+                if (
+                    filename.split(".")[-1] not in parser.EXCLUDED_FILETYPES
+                    and patch_details is not None
+                ):
                     prompt = FILE_CODE_REVIEW_PROMPT.format(
                         PULL_REQUEST_TITLE=pull_request_title,
                         PULL_REQUEST_DESC=pull_request_desc,
@@ -69,16 +87,17 @@ class CodeReviewer:
                     total_usage = self.provider.update_usage(total_usage, usage)
                     review_json = parser.extract_json(resp)
                     reviews.extend(review_json["review"])
-        review = output.create_pr_review_from_json(reviews)
-        self.logger.debug(f"Generated Review:\n {review}")
+
+        topics = self.merge_topics(reviews=reviews)
         # Share the review on pull request
         prompt_cost, completion_cost = self.provider.get_usage_cost(
             total_usage=total_usage
         )
 
         return ReviewOutput(
-            review=review,
             usage=total_usage,
+            model_name=self.provider.model,
+            topics=topics,
             cost={"prompt_cost": prompt_cost, "completion_cost": completion_cost},
         )
 
@@ -112,5 +131,47 @@ class CodeReviewer:
         return DescOutput(
             desc=body,
             usage=total_usage,
+            model_name=self.provider.model,
             cost={"prompt_cost": prompt_cost, "completion_cost": completion_cost},
         )
+
+    def merge_topics(self, reviews):
+        topics = {}
+        for review in reviews:
+            if review["topic"] in topics:
+                topics[review["topic"]].append(review)
+            else:
+                topics[review["topic"]] = [review]
+        return topics
+
+    def create_pr_review_text(self, topics):
+        markdown_title = "## Code Review\n\n"
+        markdown_output = ""
+        high_ranked_issues = 0
+        for topic, reviews in topics.items():
+            if len(reviews) > 0:
+                markdown_output += f"### {topic}\n\n"
+                for review in reviews:
+                    if review.get("confidence", "") == "critical":
+                        high_ranked_issues += 1
+                    ct = output.PR_COLLAPSIBLE_TEMPLATE.format(
+                        comment=review.get("comment", "NA"),
+                        reasoning=review.get("reasoning", "NA"),
+                        solution=review.get("solution", "NA"),
+                        confidence=review.get("confidence", "NA"),
+                        start_line=review.get("start_line", "NA"),
+                        end_line=review.get("end_line", "NA"),
+                        file_name=review.get("file_name", "NA"),
+                    )
+                    markdown_output += ct + "\n"
+        if high_ranked_issues > 0:
+            markdown_output = (
+                "❗ This review needs attention. 🚨\n\nHere are some feedback:\n\n"
+                + markdown_output
+            )
+        else:
+            markdown_output = (
+                "✅ This is a good review! 👍\n\nHere are some feedback:\n\n"
+                + markdown_output
+            )
+        return markdown_title + markdown_output
